<?php
/**
 * Jet_Search_Template_Functions class
 *
 * @package   jet-search
 * @author    Zemez
 * @license   GPL-2.0+
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Jet_Search_Template_Functions' ) ) {

	/**
	 * Define Jet_Search_Template_Functions class
	 */
	class Jet_Search_Template_Functions {

		/**
		 * Return post thumbnail.
		 *
		 * @since 1.0.0
		 * @param array  $data
		 * @param object $post
		 *
		 * @return string
		 */
		static public function get_post_thumbnail( $data, $post ) {
			$visible = filter_var( $data['thumbnail_visible'], FILTER_VALIDATE_BOOLEAN );

			if ( ! $visible ) {
				return '';
			}

			$thumbnail_format = apply_filters( 'jet-search/ajax-search/thumbnail-html', '<div class="jet-ajax-search__item-thumbnail">%s</div>' );
			$thumbnail_size   = ! empty( $data['thumbnail_size'] ) ? $data['thumbnail_size'] : 'thumbnail';
			$thumbnail_html   = get_the_post_thumbnail( $post->ID, $thumbnail_size, array( 'class' => 'jet-ajax-search__item-thumbnail-img' ) );

			if ( ! $thumbnail_html ) {
				$thumbnail_html = apply_filters(
					'jet-search/ajax-search/thumbnail-placeholder-html',
					'<div class="jet-ajax-search__item-thumbnail-placeholder"></div>'
				);
			}

			$output_html = sprintf( $thumbnail_format, $thumbnail_html );

			return $output_html;
		}

		/**
		 * Return post content.
		 *
		 * @since 1.0.0
		 * @param array  $data
		 * @param object $post
		 *
		 * @return string
		 */
		static public function get_post_content( $data, $post ) {
			$after  = '&hellip;';
			$length = ( int ) $data['post_content_length'];

			if ( 0 !== $length ) {
				$content = strip_shortcodes( $post->post_content );

				return wp_trim_words( $content, $length, $after );
			}

			return '';
		}

		/**
		 * Return product price.
		 *
		 * @since 1.1.0
		 * @param array  $data
		 * @param object $post
		 *
		 * @return string
		 */
		static public function get_product_price( $data, $post ) {
			$visible = filter_var( $data['show_product_price'], FILTER_VALIDATE_BOOLEAN );

			if ( ! $visible || 'product' !== $post->post_type || ! function_exists( 'wc' ) ) {
				return '';
			}

			$product    = wc_get_product( $post );
			$price_html = $product->get_price_html();

			if ( empty( $price_html ) ) {
				return '';
			}

			$price_format = apply_filters( 'jet-search/ajax-search/product-price-html', '<div class="jet-ajax-search__item-price"><div class="price">%s</div></div>' );

			return sprintf( $price_format, $price_html );
		}

		/**
		 * Return product rating.
		 *
		 * @since 1.1.0
		 * @param array  $data
		 * @param object $post
		 *
		 * @return string
		 */
		static public function get_product_rating( $data, $post ) {
			$visible = filter_var( $data['show_product_rating'], FILTER_VALIDATE_BOOLEAN );

			if ( ! $visible || 'product' !== $post->post_type || ! function_exists( 'wc' ) ) {
				return '';
			}

			$product = wc_get_product( $post );
			$rating  = $product->get_average_rating();

			if ( empty( $rating ) ) {
				return '';
			}
			$rating         = round( $rating, 1 );
			$floored_rating = (int) $rating;
			$rating_html    = '';
			$icon           = '&#61445;';

			for ( $stars = 1; $stars <= 5; $stars++ ) {
				if ( $stars <= $rating ) {
					$rating_html .= '<i class="jet-ajax-search__rating-star fa fa-star">' . $icon . '</i>';
				} elseif ( $floored_rating + 1 === $stars && $rating > $floored_rating ) {
					$rating_html .= '<i class="jet-ajax-search__rating-star fa fa-star jet-ajax-search__rating-star-' . ( $rating - $floored_rating ) * 10 . '">' . $icon . '</i>';
				} else {
					$rating_html .= '<i class="jet-ajax-search__rating-star fa jet-ajax-search__rating-star-empty">' . $icon . '</i>';
				}
			}

			$rating_format = apply_filters( 'jet-search/ajax-search/product-rating-html', '<div class="jet-ajax-search__item-rating">%s</div>' );

			return sprintf( $rating_format, $rating_html );
		}
	}
}
