<?php
/**
 * Jet_Search_Tools class
 *
 * @package   jet-search
 * @author    Zemez
 * @license   GPL-2.0+
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

if ( ! class_exists( 'Jet_Search_Tools' ) ) {

	/**
	 * Define Jet_Search_Tools class
	 */
	class Jet_Search_Tools {

		/**
		 * Get public post types options list
		 *
		 * @return array
		 */
		public static function get_post_types() {
			$post_types = get_post_types( array( 'show_in_nav_menus' => true ), 'objects' );

			$result = array();

			if ( empty( $post_types ) ) {
				return $result;
			}

			foreach ( $post_types as $slug => $post_type ) {
				$result[ $slug ] = $post_type->label;
			}

			return $result;
		}

		/**
		 * Get public taxonomies options list
		 *
		 * @return array
		 */
		public static function get_taxonomies() {
			$taxonomies = get_taxonomies( array( 'show_in_nav_menus' => true ), 'objects' );

			$result = array();

			if ( empty( $taxonomies ) ) {
				return $result;
			}

			foreach ( $taxonomies as $slug => $post_type ) {
				$result[ $slug ] = $post_type->label;
			}

			return $result;
		}

		/**
		 * Returns image size array in slug => name format
		 *
		 * @return  array
		 */
		public static function get_image_sizes() {

			global $_wp_additional_image_sizes;

			$sizes  = get_intermediate_image_sizes();
			$result = array();

			foreach ( $sizes as $size ) {
				if ( in_array( $size, array( 'thumbnail', 'medium', 'medium_large', 'large' ) ) ) {
					$result[ $size ] = ucwords( trim( str_replace( array( '-', '_' ), array( ' ', ' ' ), $size ) ) );
				} else {
					$result[ $size ] = sprintf(
						'%1$s (%2$sx%3$s)',
						ucwords( trim( str_replace( array( '-', '_' ), array( ' ', ' ' ), $size ) ) ),
						$_wp_additional_image_sizes[ $size ]['width'],
						$_wp_additional_image_sizes[ $size ]['height']
					);
				}
			}

			return array_merge( array( 'full' => esc_html__( 'Full', 'jet-search' ), ), $result );
		}

		/**
		 * Return available prev arrows list
		 *
		 * @return array
		 */
		public static function get_available_prev_arrows_list() {
			return apply_filters(
				'jet-search/available-nav-arrows/prev',
				array(
					'fa fa-angle-left'          => esc_html__( 'Angle', 'jet-search' ),
					'fa fa-chevron-left'        => esc_html__( 'Chevron', 'jet-search' ),
					'fa fa-angle-double-left'   => esc_html__( 'Angle Double', 'jet-search' ),
					'fa fa-arrow-left'          => esc_html__( 'Arrow', 'jet-search' ),
					'fa fa-caret-left'          => esc_html__( 'Caret', 'jet-search' ),
					'fa fa-long-arrow-left'     => esc_html__( 'Long Arrow', 'jet-search' ),
					'fa fa-arrow-circle-left'   => esc_html__( 'Arrow Circle', 'jet-search' ),
					'fa fa-chevron-circle-left' => esc_html__( 'Chevron Circle', 'jet-search' ),
					'fa fa-caret-square-o-left' => esc_html__( 'Caret Square', 'jet-search' ),
				)
			);
		}

		/**
		 * Return available prev arrows list
		 *
		 * @return array
		 */
		public static function get_available_next_arrows_list() {
			return apply_filters(
				'jet-search/available-nav-arrows/next',
				array(
					'fa fa-angle-right'          => esc_html__( 'Angle', 'jet-search' ),
					'fa fa-chevron-right'        => esc_html__( 'Chevron', 'jet-search' ),
					'fa fa-angle-double-right'   => esc_html__( 'Angle Double', 'jet-search' ),
					'fa fa-arrow-right'          => esc_html__( 'Arrow', 'jet-search' ),
					'fa fa-caret-right'          => esc_html__( 'Caret', 'jet-search' ),
					'fa fa-long-arrow-right'     => esc_html__( 'Long Arrow', 'jet-search' ),
					'fa fa-arrow-circle-right'   => esc_html__( 'Arrow Circle', 'jet-search' ),
					'fa fa-chevron-circle-right' => esc_html__( 'Chevron Circle', 'jet-search' ),
					'fa fa-caret-square-o-right' => esc_html__( 'Caret Square', 'jet-search' ),
				)
			);
		}
	}
}
